package com.contextspace;

import java.io.File;
import java.util.Collection;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Expression;
import javax.persistence.criteria.Join;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

import static org.junit.Assert.*;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import com.contextspace.objectdb_example.model.Attribute;
import com.contextspace.objectdb_example.model.AttributeTypes;
import com.contextspace.objectdb_example.model.CDI;

public class ExampleTest {
 
    private static EntityManagerFactory emf;
    private static EntityManager em;
    private static CDI createdCdi6;
    
    @BeforeClass
    public static void onceStart() throws Exception {
        emf = Persistence.createEntityManagerFactory("objectdb/db/test.odb");
        em = emf.createEntityManager();
 
        em.getTransaction().begin();
        
        CDI createdCdi1 = createCDI("baffleck", "Ben", "Geza", "Affleck");
        em.persist(createdCdi1);
        CDI createdCdi2 = createCDI("jalba", "Jessica", "Marie", "Alba");
        em.persist(createdCdi2);
        CDI createdCdi3 = createCDI("kbeckinsale", "Kate", null, "Beckinsale");
        em.persist(createdCdi3);
        CDI createdCdi4 = createCDI("abaldwin", "Alex", "Rae", "Baldwin");
        em.persist(createdCdi4);
        CDI createdCdi5 = createCDI("sbaldwin", "Stephen", "Andrew", "Baldwin");
        em.persist(createdCdi5);
        createdCdi6 = createCDI("wbaldwin", "William", null, "Baldwin");
        em.persist(createdCdi6);
        
        em.getTransaction().commit();
        
    }
    
    @AfterClass
    public static void onceEnd() throws Exception {
        
        if(em != null) {
            em.close();
        }
        
        if(emf != null) {
            emf.close();
        }
        
        File f1 = new File("objectdb/db/test.odb");
        if(f1.exists()) {
            f1.delete();
        }
    }
    
    protected static CDI createCDI(String username, String first, String middle, String last) {
        CDI cdi = new CDI();
        cdi.setCdiId(username);
        Attribute firstname = new Attribute(AttributeTypes.firstname, first);
        cdi.addAttribute(firstname);
        Attribute lastname = new Attribute(AttributeTypes.lastname, last);
        cdi.addAttribute(lastname);
        if(middle != null) {
            Attribute middlename = new Attribute(AttributeTypes.middlenames, middle);
            cdi.addAttribute(middlename);
        }
        Attribute title = new Attribute(AttributeTypes.title, "Mr");
        cdi.addAttribute(title);
      
        return cdi;
    }
    
    
    @Test
    public void queryAttribute1() throws Exception {
        
        CriteriaBuilder cb = em.getCriteriaBuilder();
        CriteriaQuery<CDI> q = cb.createQuery(CDI.class);
        Root<CDI> cdi = q.from(CDI.class);
        
        Join<CDI, Attribute> attributeJoin = cdi.join("attributeList");
        Predicate pred = 
                cb.and(
                    cb.equal(attributeJoin.get("valueAsString"), "Alex"), 
                    cb.equal(attributeJoin.get("name"), "firstname")
                );
        
        q.select(cdi).where(pred);
        TypedQuery<CDI> typedQuery = em.createQuery(q);
        List<CDI> results = typedQuery.getResultList();
        System.out.println("RESULT: " + results);
        assertEquals(1, results.size());
        assertEquals("Alex", results.get(0).getAttribute("firstname").get(0).getValue());
    }
    
    
    @Test
    public void queryAttribute2() throws Exception {
        
        CriteriaBuilder cb = em.getCriteriaBuilder();
        CriteriaQuery<CDI> q = cb.createQuery(CDI.class);
        Root<CDI> cdi = q.from(CDI.class);
        
        Join<CDI, Attribute> attributeJoin = cdi.join("attributeList");
        Join<CDI, Attribute> attributeJoin2 = cdi.join("attributeList");
        Predicate pred = 
            cb.and(
                cb.and(
                    cb.equal(attributeJoin.get("valueAsString"), "Alex"), 
                    cb.equal(attributeJoin.get("name"), "firstname")
                ),
                cb.and(
                    cb.equal(attributeJoin2.get("valueAsString"), "Baldwin"), 
                    cb.equal(attributeJoin2.get("name"), "lastname")
                )
            );
        
        q.select(cdi).where(pred);
        TypedQuery<CDI> typedQuery = em.createQuery(q);
        List<CDI> results = typedQuery.getResultList();
        System.out.println("RESULT: " + results);
        assertEquals(1, results.size());
        assertEquals("Alex", results.get(0).getAttribute("firstname").get(0).getValue());
    }
    
    @Test
    public void queryAttribute3() throws Exception {
        
        CriteriaBuilder cb = em.getCriteriaBuilder();
        CriteriaQuery<CDI> q = cb.createQuery(CDI.class);
        Root<CDI> cdi = q.from(CDI.class);
        
        Join<CDI, Attribute> attributeJoin = cdi.join("attributeList");
        Predicate pred = 
            cb.or(
                cb.and(
                    cb.equal(attributeJoin.get("valueAsString"), "Alex"), 
                    cb.equal(attributeJoin.get("name"), "firstname")
                ),
                cb.and(
                    cb.equal(attributeJoin.get("valueAsString"), "William"), 
                    cb.equal(attributeJoin.get("name"), "firstname")
                )
            );
        
        q.select(cdi).where(pred);
        TypedQuery<CDI> typedQuery = em.createQuery(q);
        List<CDI> results = typedQuery.getResultList();
        System.out.println("RESULT: " + results);
        assertEquals(2, results.size());
    }
    
    @Test
    public void queryAttribute4() throws Exception {
        
        TypedQuery<CDI> query = em.createQuery("SELECT DISTINCT t FROM com.contextspace.objectdb_example.model.CDI t JOIN t.attributeList attr JOIN t.attributeList attr2 WHERE " +
                "attr.value='Alex' and attr.name='firstname' and attr2.value='Baldwin' and attr2.name='lastname'", CDI.class);

        List<CDI> results = query.getResultList();
        System.out.println("RESULT: " + results);
        assertEquals(1, results.size());
    }
    
    @Test
    public void queryAttribute5() throws Exception {
        
        TypedQuery<CDI> query = em.createQuery("SELECT DISTINCT t FROM com.contextspace.objectdb_example.model.CDI t JOIN t.attributeList attr WHERE " +
                "(attr.value='Alex' and attr.name='firstname') or (attr.value='William' and attr.name='firstname')", CDI.class);

        List<CDI> results = query.getResultList();
        System.out.println("RESULT: " + results);
        assertEquals(2, results.size());
    }
        
}
